// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "SubSceneCameraAlign.hpp"

using namespace Dawn;

SubSceneCameraAlign::SubSceneCameraAlign(SceneItem *i) :
  SceneItemComponent(i),
  camera(nullptr),
  renderTarget(nullptr)
{
}

void SubSceneCameraAlign::realign() {
  float_t diff;
  if(this->camera == nullptr) return;
  if(this->renderTarget == nullptr) return;

  float_t ratio = this->renderTarget->getWidth() / this->renderTarget->getHeight();
  float_t myRatio = this->camera->getAspect();
  
  this->camera->type = CAMERA_TYPE_ORTHONOGRAPHIC;
  this->camera->transform->lookAt(glm::vec3(0, 0, 10), glm::vec3(0, 0, 0));

  if(ratio > myRatio) {
    // My Ratio is narrower
    this->camera->orthoLeft = 0;
    this->camera->orthoRight = this->renderTarget->getWidth();
    diff = (this->renderTarget->getHeight() - (this->renderTarget->getWidth() / myRatio)) / 2.0f;
    this->camera->orthoTop = this->renderTarget->getHeight() - diff;
    this->camera->orthoBottom = diff;
  } else {
    // My ratio is wider
    this->camera->orthoBottom = 0;
    this->camera->orthoTop = this->renderTarget->getHeight();
    diff = (this->renderTarget->getWidth() - (this->renderTarget->getHeight() * myRatio)) / 2.0f;
    this->camera->orthoLeft = diff;
    this->camera->orthoRight = this->renderTarget->getWidth() - diff;
  }
}

void SubSceneCameraAlign::onStart() {
  auto cameraEffect = useEffectWithTeardown([&]{
    if(camera == nullptr) return evtCameraResized = [&] {};

    this->realign();

    return evtCameraResized = useEvent([&](float_t w, float_t h){
      this->realign();
    }, this->camera->eventRenderTargetResized);
  }, this->camera);

  auto renderEffect = useEffectWithTeardown([&]{
    if(renderTarget == nullptr) return evtRenderResized = [&]{};

    this->realign();

    return evtRenderResized = useEventLegacy([&](RenderTarget *t, float_t w, float_t h) {
      this->realign();
    }, renderTarget->eventRenderTargetResized);
  }, this->renderTarget);

  cameraEffect();
  renderEffect();
}