// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "PlayerController.hpp"
#include "game/DawnGame.hpp"

using namespace Dawn;

PlayerController::PlayerController(SceneItem *item) : SceneItemComponent(item) {

}

std::vector<SceneItemComponent*> PlayerController::getDependencies() {
  return {
    (this->entityMove = item->getComponent<EntityMove>()),
    (this->entitySwordAttack = item->getComponent<EntitySwordAttack>())
  };
}

void PlayerController::onStart() {
  assertNotNull(this->entityMove);

  useEvent([&](float_t delta){
    // Movement
    this->entityMove->direction = getGame()->inputManager.getAxis2D(
      INPUT_BIND_NEGATIVE_X, INPUT_BIND_POSITIVE_X,
      INPUT_BIND_NEGATIVE_Y, INPUT_BIND_POSITIVE_Y
    );

    if(this->entitySwordAttack != nullptr && getGame()->inputManager.isPressed(INPUT_BIND_ACCEPT)) {
      this->entitySwordAttack->attack();
    }
  }, getScene()->eventSceneUpdate);
}